"""Aliases for the default django widgets.

In case of Django ever moving them around, only this file will need updating. No new migrations."""

# pylint: disable=unused-import
from django.forms import Select, NumberInput
